package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.server.common.FilePath;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDConfigSign;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.*;
import com.malk.utils.UtilFile;
import com.malk.utils.UtilHttp;
import com.malk.utils.UtilList;
import com.malk.utils.UtilMap;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
@Slf4j
public class DDImplService implements DDService {

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    @Autowired
    private DDClient_Attendance ddClient_attendance;

    @Autowired
    private DDClient_Contacts ddClient_contacts;

    @Autowired
    private DDClient_Storage ddClient_storage;

    @Autowired
    private FilePath filePath;

    @Autowired
    private DDConf ddConf;

    @Autowired
    private DDClient ddClient;

    /**
     * 新发起审批15s内不允许撤销, 异步执行 [审批同意/拒绝只能通过节点操作, 系统无法直接介入]   -- 异步需要中转一层进行触发, client为原子接口
     */
    @Async
    @Override
    @SneakyThrows
    public void terminateRunningApprove_async(String access_token, String processInstanceId, boolean isSystem, String subject, String description, String operatingUserId, String noteUserId) {
        ddClient_workflow.createTBTask(access_token, noteUserId, subject, description, new Date().getTime() + 16000, Arrays.asList(noteUserId), null, null, false, 20, null);
        Thread.sleep(16000);
        ddClient_workflow.terminateRunningApprove(access_token, processInstanceId, isSystem, description, operatingUserId);
    }

    /**
     * 钉钉查询假期余额返回是记录, 按照消失/天单位计算该假期类型下的余额
     */
    @Override
    public Map queryVacationQuota_balance(String access_token, String op_userid, String leave_code, String userids, int offset, int size) {
        List<Map> records = ddClient_attendance.queryVacationQuota_all(access_token, op_userid, leave_code, userids, offset, size);
        if (ObjectUtil.isNull(records)) {
            return null;
        }
        AtomicInteger balance = new AtomicInteger();
        AtomicReference<String> unit = new AtomicReference<>("");
        records.forEach(item -> {
            if (ObjectUtil.isNotNull(item.get("quota_num_per_hour"))) {
                balance.set(balance.get() + Integer.valueOf(String.valueOf(item.get("quota_num_per_hour"))) - Integer.valueOf(String.valueOf(item.get("used_num_per_hour"))));
                unit.set("小时");
            } else {
                balance.set(balance.get() + (Integer.valueOf(String.valueOf(item.get("quota_num_per_day"))) - Integer.valueOf(String.valueOf(item.get("used_num_per_day")))));
                unit.set("天");
            }
        });
        Map result = new HashMap();
        result.put("balance", balance.get() / 100f);
        result.put("unit", unit);
        return result;
    }

    /**
     * 上传审批附件
     *
     * @param urlFile      远程文件地址
     * @param fileNamePure 文件名称[后缀自动通过地址获取]
     */
    @Override
    public Map uploadFileFormUrlForOverflow(String accessToken, String userId, String urlFile, String fileNamePure) {
        // 下载到本地
        String fileName = fileNamePure + urlFile.substring(urlFile.lastIndexOf(".")).toLowerCase();
        File file = UtilFile.mkdirIfNot(fileName, filePath.getPath().getFile());
        UtilHttp.doDownload(urlFile, file);
        return uploadFileFormUrlForOverflow(accessToken, userId, file.getAbsolutePath());
    }

    /**
     * 上传审批附件
     *
     * @param filePath 已存在文件, 本地file绝对路径
     */
    @Override
    public Map uploadFileFormUrlForOverflow(String accessToken, String userId, String filePath) {
        // 获取用户unionId
        String unionId = String.valueOf(ddClient_contacts.getUserInfoById(accessToken, userId).get("unionid"));
        // 获取储存空间
        String spaceId = ddClient_storage.getSpacesInfos(accessToken, userId, null);
        // 添加空间用户权限
        List<Map> members = Arrays.asList(UtilMap.map("type, id, corpId", "USER", unionId, ddConf.getCorpId()));
        ddClient_storage.addSpacePermissions(accessToken, spaceId, "0", unionId, "EDITOR", members, null);
        // 获取空间上传信息
        DDR_New ddr_new = ddClient_storage.getUploadInfos(accessToken, spaceId, unionId, null);
        // 执行文件上传
        String resourceUrl = ((List<String>) ddr_new.getHeaderSignatureInfo().get("resourceUrls")).get(0);
        ddClient_storage.uploadFiles(resourceUrl, (Map<String, String>) ddr_new.getHeaderSignatureInfo().get("headers"), filePath);
        // 提交文件
        return ddClient_storage.commitFiles(accessToken, spaceId, unionId, ddr_new.getUploadKey(), String.valueOf(UtilList.getLast(filePath.split("/"))), "0", null);
    }

    /**
     * 上传钉盘文件
     *
     * @param urlFile 远程文件地址
     */
    @Override
    public Map uploadFileFormUrlForDingDrive(String accessToken, String unionId, String spaceId, String parentId, String urlFile, String fileNamePure) {
        // 下载到本地 [兼容中文]
        // String fileName = URLDecoder.decode(String.valueOf(UtilList.getLast(urlFile.split("/"))));
        File file = UtilFile.mkdirIfNot(fileNamePure, filePath.getPath().getFile());
        UtilHttp.doDownload(urlFile, file);
        // 获取空间上传信息
        DDR_New ddr_new = ddClient_storage.getUploadInfos(accessToken, spaceId, unionId, null);
        // 执行文件上传
        String resourceUrl = ((List<String>) ddr_new.getHeaderSignatureInfo().get("resourceUrls")).get(0);
        ddClient_storage.uploadFiles(resourceUrl, (Map<String, String>) ddr_new.getHeaderSignatureInfo().get("headers"), file.getAbsolutePath());
        // 提交文件
        Map dentry = ddClient_storage.commitFiles(accessToken, spaceId, unionId, ddr_new.getUploadKey(), fileNamePure, parentId, null);
        // 删除本地临时文件
        UtilFile.deleteFile(file.getAbsolutePath());
        return dentry;
    }

    /**
     * 判断员工是否在指定部门
     */
    @Override
    public boolean matchDepartment(String access_token, String userId, List<Long> deptIds) {
        List<Number> deptIdList = (List<Number>) ddClient_contacts.getUserInfoById(access_token, userId).get("dept_id_list");
        boolean isMatch = false;
        // 兼容多部门场景
        for (Number deptId : deptIdList) {
            // ppExt: 不要直接使用 Number 作为类型, 不同基本类型比较值时会有偏差 [可以作为父类接受数据, 避免直接强制类型转换错误, 再进行基本类型处理]
            isMatch = _matchDepartment(access_token, deptId.longValue(), deptIds);
            if (isMatch) {
                break;
            }
        }
        return isMatch;
    }

    /// 递归: 判断员工是否在指定部门
    boolean _matchDepartment(String access_token, long dept_id, List<Long> deptIds) {
        boolean isMatch = false;
        // 判断入参 [同样作为递归出口]
        if (dept_id == DDConf.TOP_DEPARTMENT) {
            return false;
        }
        if (deptIds.contains(dept_id)) {
            isMatch = true;
        }
        // 递归出口 [查询上级部门匹配]
        if (!isMatch) {
            Map deptInfo = ddClient_contacts.getDepartmentInfo(access_token, dept_id);
            long parentId = UtilMap.getLong(deptInfo, "parent_id");
            return _matchDepartment(access_token, parentId, deptIds);
        }
        return isMatch;
    }

    /**
     * 获取员工所属部门全路径
     */
    @Override
    public List<Map> getUserDepartmentHierarchy(String access_token, String userId) {
        // PRD: 一个人存在多个部门默认取第一个
        List<Number> deptIdList = (List<Number>) ((List<Map>) ddClient_contacts.listParentByUser(access_token, userId).get("parent_list")).get(0).get("parent_dept_id_list");
        List<Map> deptInfo = new ArrayList();
        // Number 仅仅作为数据类型声明, 避免比较类型不一致导致判定问题
        for (Number deptId : deptIdList) {
            if (deptId.longValue() == DDConf.TOP_DEPARTMENT) {
                continue;
            }
            deptInfo.add(UtilMap.map("id, name", deptId, ddClient_contacts.getDepartmentInfo(access_token, deptId.longValue()).get("name")));
        }
        Collections.reverse(deptInfo);
        return deptInfo;
    }

    /**
     * 获取员工所属部门路径层级 [名称拼接]
     */
    @Override
    public String getUserDepartmentHierarchyJoin(String access_token, String userId, String delimiter) {
        return String.join(delimiter, getUserDepartmentHierarchy(access_token, userId).stream().map(dept -> dept.get("name").toString()).collect(Collectors.toList()));
    }

    /**
     * jsApi 注册
     */
    @Override
    public Map registerJsApi(String url, String nonceStr) {
        String jsTicket = ddClient.getJsApiTicket(ddClient.getAccessToken());
        long timeStamp = new Date().getTime();
        String signature = DDConfigSign.sign(jsTicket, nonceStr, timeStamp, url);
        return UtilMap.map("nonceStr, agentId, timeStamp, corpId, signature", nonceStr, ddConf.getAgentId(), timeStamp, ddConf.getCorpId(), signature);
    }

    /**
     * 免登code获取用户信息
     */
    @Override
    public Map getUserInfoByCode(String code) {
        Map rsp = ddClient.getUserInfoByCode(ddClient.getAccessToken(), code);
        return ddClient_contacts.getUserInfoById(ddClient.getAccessToken(), rsp.get("userid").toString());
    }

    @Override
    public String getFile_new(String accessToken, String unionId, String spaceId, String parentId, String fileId, String fileNamePure) {
        Date currentTime = new Date();

        String saveFilePath =  "files_OA"; // 替换为你想要保存文件的路径
        String delurl = "";
        OutputStreamWriter out = null;
        // 定义输入流
        BufferedReader br = null;
        String result = "";
        FileOutputStream outfile = null;
        try {
            // 统一资源
            URL url = new URL("https://www.h3yun.com/Api/DownloadBizObjectFile");

            // 打开和url之间的连接
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();

            // 设定请求的方法为"POST"，默认是GET
            // post与get的不同之处在于post的参数不是放在URL字串里面，而是放在http请求的正文内。
            conn.setRequestMethod("POST");

            // 设置30秒连接超时
            conn.setConnectTimeout(30000);
            // 设置30秒读取超时
            conn.setReadTimeout(30000);

            // 设置是否向httpUrlConnection输出，因为这个是post请求，参数要放在http正文内，因此需要设为true, 默认情况下是false;
            conn.setDoOutput(true);
            // 设置是否从httpUrlConnection读入，默认情况下是true;
            conn.setDoInput(true);

            // Post请求不能使用缓存
            conn.setUseCaches(false);

            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive"); // 维持长链接
            conn.setRequestProperty("Content-Type", "application/json;charset=utf-8");
            // 必须要有的身份认证参数 在组织 界面 钉钉右上角 我的头像点击 后 系统管理》系统集成中有这两个字段
            conn.setRequestProperty("EngineCode", "jd4fneb66qz2gfvau6svhkxk1");
            conn.setRequestProperty("EngineSecret", "F8K2W/09HZpQGiN0IgRNNi+EJKCFz9Zu6xTNsz6yjCpsAS0+tbSEeQ==");
            // 连接，从上述url.openConnection()至此的配置必须要在connect之前完成，
            conn.connect();
            String jsonInputString = "{\"EngineCode\":\"jd4fneb66qz2gfvau6svhkxk1\", \"attachmentId\":\"" + fileId + "\"}";

            /**
             * 下面的三句代码，就是调用第三方http接口
             */
            // 获取URLConnection对象对应的输出流
            // 此处getOutputStream会隐含的进行connect(即：如同调用上面的connect()方法，所以在开发中不调用上述的connect()也可以)。
            out = new OutputStreamWriter(conn.getOutputStream(), "UTF-8");
            // 发送请求参数即数据
            out.write(jsonInputString);
            // flush输出流的缓冲
            out.flush();

            /**
             * 下面的代码相当于，获取调用第三方http接口后返回的结果
             */
            // 获取URLConnection对象对应的输入流
            InputStream is = conn.getInputStream();

            // 获取文件类型 例如：返回的格式是：attachment;filename=1.png
            String headerField = conn.getHeaderField("Content-Disposition");
            headerField = new String(headerField.getBytes("GB2312"));
            headerField = new String(headerField.getBytes("utf-8"));
            // 截取得到正确的文件名
            String filename = substringGetType(headerField);
            byte[] NEWdata = getByteData(is);

            // 建立存储的目录、保存的文件名
            File file = new File(saveFilePath);
            if (!file.exists()) {
                file.mkdirs();
            }
            // 修改文件名
            File res = new File(file + File.separator + fileNamePure);
            // 写入输出流
            outfile = new FileOutputStream(res);
            outfile.write(NEWdata);
            System.out.println("下载成功!");
            // 关闭流
            is.close();
            // 断开连接，disconnect是在底层tcp socket链接空闲时才切断，如果正在被其他线程使用就不切断。
            conn.disconnect();

            // 获取空间上传信息
            DDR_New ddr_new = ddClient_storage.getUploadInfos(accessToken, spaceId, unionId, null);
            // 执行文件上传
            String resourceUrl = ((List<String>) ddr_new.getHeaderSignatureInfo().get("resourceUrls")).get(0);
            ddClient_storage.uploadFiles(resourceUrl, (Map<String, String>) ddr_new.getHeaderSignatureInfo().get("headers"), file + File.separator + fileNamePure);
            // 提交文件
            Map dentry = ddClient_storage.commitFiles(accessToken, spaceId, unionId, ddr_new.getUploadKey(), fileNamePure, parentId, null);
            if (out != null) {
                out.close();
                outfile.close();


            }

            if (br != null) {
                br.close();
            }


            //删除文件
            delurl = file + File.separator + fileNamePure;

            File directory = new File(saveFilePath);
            // 获取目录下的所有文件和目录
            File[] files = directory.listFiles();

            // 确保目录存在且不是文件本身
            if (files != null && directory.isDirectory()) {
                for (File files_A : files) {
                    // 删除文件
                    if (files_A.isFile()) {
                        System.out.println("Deleting file: " + files_A.getName());
                        files_A.delete();
                    }
                }
            }


        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("下载失败!");
        }
//            if (delurl!=""){
//
//                deleteFile(delurl);
//                System.out.println("删除成功!");
//            }

        return accessToken;

    }

    private static byte[] getByteData(InputStream in) throws IOException {
        byte[] b = new byte[1024];
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        int len = 0;
        while ((len = in.read(b)) != -1) {
            bos.write(b, 0, len);
        }
        if (null != bos) {
            bos.close();
        }
        return bos.toByteArray();
    }

    /**
     * @Title: substringGetPng
     * @Description: 截取字符串得到正确的文件类型
     */
    public static String substringGetType(String str) {
        // 获得第=号的位置
        int index = str.indexOf("=");
        String result = str.substring(index + 1);
        // 输出结果
        //System.out.println("下载文件名:" + result);
        return result;
    }

    private void deleteFile(String url) {

        File file = new File(url);
        if (file.isFile() && file.exists()) {
            file.delete();
        }
    }
}
