package com.malk.service.h3yun.impl;

import com.malk.server.common.FilePath;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.h3yun.CYClient;
import com.malk.utils.UtilFile;
import com.malk.utils.UtilHttp;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CYImplClient implements CYClient {

    /// 请求路径
    private String getRequestUrl(String path) {
        return "https://api.dingtalk.com/v1.0/h3yun" + path;
    }
    private static String  getRequestUrl_H3yun= "https://www.h3yun.com/OpenApi/Invoke";
    /**
     * 获取附件临时免登地址
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-the-temporary-attachment-free-address
     */
    @Override
    public String getTemporaryUrls(String accessToken, String attachmentId) {
        Map param = UtilMap.map("attachmentId", attachmentId);
        DDR_New cyr = DDR_New.doGet(getRequestUrl("/attachments/temporaryUrls"), DDConf.initTokenHeader(accessToken), param);
        return UtilMap.getString(((Map) cyr.getData()), "attachmentUrl");
    }

    @Autowired
    private FilePath filePath;

    /**
     * 储存氚云附件到本地, 保留文件名称与格式
     */
    @Override
    public File saveFromTemporaryUrl(String accessToken, Map attachment) {
        String fileUrl = getTemporaryUrls(accessToken, UtilMap.getString(attachment, "FileId"));
        String fileName = UtilMap.getString(attachment, "FileName");
        File file = UtilFile.mkdirIfNot(fileName, filePath.getPath().getFile());
        UtilHttp.doDownload(fileUrl, file);
        return file;
    }

    /**
     * 查询表单数据
     */
    @Override
    public List<Map> queryData(String accessToken, String schemaCode, Map extInfo) {

        extInfo = UtilMap.empty(extInfo);
        UtilMap.put(extInfo, "pageSize", 500);
        UtilMap.put(extInfo, "pageNumber", 1);
        extInfo.put("schemaCode", schemaCode);
        DDR_New cyr = DDR_New.doPost(getRequestUrl("/forms/instances/search"), DDConf.initTokenHeader(accessToken), null, extInfo);
        return (List<Map>) ((Map) cyr.getData()).get("bizObjects");
    }

    /**
     * 创建流程数据
     */
    @Override
    public Map operateData(String accessToken, String schemaCode, Map extInfo) {

        Map body = UtilMap.map("schemaCode", schemaCode);

        String path = "/forms/instances";
        if (extInfo.containsKey("bizObjectId")) {
            path = "/processes/instances";
        }
        UtilMap.putAll(body, extInfo);
        DDR_New cyr = DDR_New.doPost(getRequestUrl(path), DDConf.initTokenHeader(accessToken), null, body);
        return (Map) cyr.getData();
    }

    /**
     * 获取用户数据
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-user-data
     */
    @Override
    public List<Map> getUserInfoList(String accessToken, String departmentId, boolean isRecursive) {
        Map param = UtilMap.map("departmentId, isRecursive", departmentId, isRecursive);
        DDR_New cyr = DDR_New.doGet(getRequestUrl("/users"), DDConf.initTokenHeader(accessToken), param);
        return (List<Map>) cyr.getData();
    }

    /**
     * 获取组织数据
     */
    @Override
    public List<Map> getDepartmentInfoList(String accessToken, String departmentId) {
        Map param = UtilMap.putNotEmpty(UtilMap.empty(), "departmentId", departmentId);
        DDR_New cyr = DDR_New.doGet(getRequestUrl("/departments"), DDConf.initTokenHeader(accessToken), param);
        return (List<Map>) cyr.getData();
    }
//案例，未完善
    @Override
    public void SelectOne(Map<String, Object> paramMap) {

    }

    @Override
    public void SelectMore(Map<String, Object> paramMap) {

    }

    @Override
    public void CreateBusinessData(Map<String, Object> paramMap) {

    }

    @Override
    public void UpdateBusinessData(Map<String, Object> paramMap) {

    }


    public static void doPost(String pathUrl, String data) {
        OutputStreamWriter out = null;
        BufferedReader br = null;
        String result = "";

        try {
            URL url = new URL(pathUrl);
            HttpURLConnection conn = (HttpURLConnection)url.openConnection();
            conn.setRequestMethod("POST");
            conn.setConnectTimeout(30000);
            conn.setReadTimeout(30000);
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("Content-Type", "application/json;charset=utf-8");
            conn.setRequestProperty("EngineCode", "fs1tkeu2ap4kb4hp");
            conn.setRequestProperty("EngineSecret", "Ed0JouQbydoe1seTTuETeqK7VbJqTX2jEWOUCnnfgZc0kzLg18nZFw==");
            conn.connect();
            out = new OutputStreamWriter(conn.getOutputStream(), "UTF-8");
            out.write(data);
            out.flush();
            InputStream is = conn.getInputStream();
            br = new BufferedReader(new InputStreamReader(is));

            for(String str = ""; (str = br.readLine()) != null; result = result + str) {
            }

            System.out.println(result);
            is.close();
            conn.disconnect();
        } catch (Exception var17) {
            var17.printStackTrace();
        } finally {
            try {
                if (out != null) {
                    out.close();
                }

                if (br != null) {
                    br.close();
                }
            } catch (IOException var16) {
                var16.printStackTrace();
            }

        }

    }

    public static void doGet(String pathUrl,String EngineCode,String EngineSecret) {
        BufferedReader br = null;
        String result = "";

        try {
            URL url = new URL(pathUrl);
            HttpURLConnection conn = (HttpURLConnection)url.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(30000);
            conn.setReadTimeout(30000);
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");
            conn.setRequestProperty("EngineCode", EngineCode);
            conn.setRequestProperty("EngineSecret", EngineSecret);
            conn.connect();
            InputStream is = conn.getInputStream();
            br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

            for(String str = ""; (str = br.readLine()) != null; result = result + str) {
            }

            System.out.println(result);
            is.close();
            conn.disconnect();
        } catch (Exception var15) {
            var15.printStackTrace();
        } finally {
            try {
                if (br != null) {
                    br.close();
                }
            } catch (IOException var14) {
                var14.printStackTrace();
            }

        }

    }
}
